// Copyright 2020 Esri.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import UIKit
import ArcGIS

class ViewController: UIViewController {
    @IBOutlet weak var mapView: AGSMapView!

    override func viewDidLoad() {
        super.viewDidLoad()
        setupMap()

        setupSearchBar()

        setupGraphics()

    }

    // MARK: - Map
    
    private func setupMap() {
        mapView.map = AGSMap(basemapStyle: .arcGISTopographic)
        mapView.setViewpoint(
            AGSViewpoint(
                latitude: 34.02700,
                longitude: -118.80543,
                scale: 144447.638572
            )
        )
    }
    
    // MARK: - Search Bar

    private func setupSearchBar() {
        navigationItem.titleView = {
            let searchBar = UISearchBar()
            searchBar.delegate = self
            searchBar.showsCancelButton = true
            searchBar.placeholder = "Search for an address"
            return searchBar
        }()
    }

    
    // MARK: - Graphics

    private func setupGraphics() {
        let graphicsOverlay = AGSGraphicsOverlay()
        graphicsOverlay.graphics.addObjects(from: [textGraphic, markerGraphic])
        mapView.graphicsOverlays.add(graphicsOverlay)
    }

    private let textGraphic: AGSGraphic = {
        let textSymbol = AGSTextSymbol(
            text: "",
            color: .black,
            size: 14,
            horizontalAlignment: .center,
            verticalAlignment: .bottom
        )
        return AGSGraphic(geometry: nil, symbol: textSymbol)
    }()

    private let markerGraphic: AGSGraphic = {
        let markerSymbol = AGSSimpleMarkerSymbol(
            style: .square,
            color: .red,
            size: 12
        )
        return AGSGraphic(geometry: nil, symbol: markerSymbol)
    }()

    // MARK: - Result

    enum ResultStatus {
        case none
        case result(String, AGSPoint)
    }

    var status: ResultStatus = .none {
        didSet {
            switch status {
            case .none:
                (textGraphic.symbol as! AGSTextSymbol).text = ""
                textGraphic.geometry = nil
                markerGraphic.geometry = nil
            case .result(let title, let location):
                (textGraphic.symbol as! AGSTextSymbol).text = title
                textGraphic.geometry = location
                markerGraphic.geometry = location
            }
        }
    }

    // MARK: - Geocoding

    private let locator = AGSLocatorTask(
        url: URL(string: "https://geocode-api.arcgis.com/arcgis/rest/services/World/GeocodeServer")!
    )

    private var currentGeocodeOperation: AGSCancelable?

    private func geocode(with searchText: String) {
        currentGeocodeOperation?.cancel()

        let parameters: AGSGeocodeParameters = {
            let parameters = AGSGeocodeParameters()
            parameters.resultAttributeNames = ["*"]
            parameters.maxResults = 1
            parameters.outputSpatialReference = mapView.spatialReference
            return parameters
        }()

        currentGeocodeOperation = self.locator.geocode(withSearchText: searchText, parameters: parameters) { [weak self] (results, error) in
            guard let self = self else { return }
            if let error = error {
                self.status = .none
                print(error.localizedDescription)
                return
            } else if let firstResult = results?.first,
                      let extent = firstResult.extent,
                      let location = firstResult.displayLocation {
                self.status = .result(firstResult.label, location)
                self.mapView.setViewpointGeometry(extent)
            } else {
                self.status = .none
                print("No results found for \(searchText).")
            }
        }

    }

}

extension ViewController: UISearchBarDelegate {

    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        searchBar.resignFirstResponder()
        guard let searchText = searchBar.text, !searchText.isEmpty else {
            print("Nothing to search!")
            return
        }
        geocode(with: searchText)
    }

    func searchBarCancelButtonClicked(_ searchBar: UISearchBar) {
        searchBar.resignFirstResponder()
    }

}


